@@ -23,6 +23,8 @@ class ScreenLogicWarning(Exception):
 
 
 class MESSAGE:
+    COM_MAX_RETRIES = 2
+    COM_RETRY_WAIT = 2
     COM_TIMEOUT = 5
     HEADER_FORMAT = "<HHI"
     HEADER_LENGTH = struct.calcsize(HEADER_FORMAT)
@@ -32,10 +34,12 @@ class MESSAGE:
 # ANSWERS are QUERY + 1
 class CODE:
     MSG_CODE_1 = 0
-    UNKNOWN_ANSWER = 13
+    ERROR_LOGIN_REJECTED = 13
     CHALLENGE_QUERY = 14
     PING_QUERY = 16
     LOCALLOGIN_QUERY = 27
+    ERROR_INVALID_REQUEST = 30
+    ERROR_BAD_PARAMETER = 31  # Actually bad parameter?
     VERSION_QUERY = 8120
     WEATHER_FORECAST_CHANGED = 9806
     WEATHER_FORECAST_QUERY = 9807
