@@ -6,7 +6,9 @@ from .gateway import async_request_gateway_version
 from .heat import async_request_set_heat_mode, async_request_set_heat_setpoint
 from .lights import async_request_pool_lights_command
 from .login import async_connect_to_gateway
+from .ping import async_request_ping
 from .pump import async_request_pump_status
 from .status import async_request_pool_status
 from .chemistry import async_request_chemistry, async_request_set_chem_data
 from .scg import async_request_scg_config, async_request_set_scg_config
+from .request import async_make_request
